/*
 * Copyright (C) 2000-2004 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */

/**
 * \file include/arch/btnode2/btn-hardware.h
 *
 * 26.05.2004 Martin Hinz <btnode@hinz.ch>
 * 
 * @brief Platform dependent port connection for AVR ATmega128(L) and
 * BTnode Rev2_2.
 *
 * If you have to distinguish between BTnode rev2 and BTnode rev3 in
 * your program code use following if on def:\n
\verbatim
 #if defined(__BTNODE3__)
     program code for BTnode rev3
 #else
     program code for everything else (BTnode rev2, STK500, linux, ...)
 #endif 
\endverbatim
 */

// Extended Memory
#ifndef __NO_XMEM__
#define NUTXMEM_START 0x1100
#define NUTXMEM_SIZE  0xEF00
#endif

// CPU Clock Frequency
#define CPU_FREQ 7372800

// CPU Type
#define MCU_ATMEGA128

// VCC in mV
#define VCC_mV 3300

//Ram port definitions
#define RAM_CTR_A_PORT PORTE
#define RAM_CTR_A_PIN 7
#define RAM_CTR_A_DDR DDRE
#define RAM_CTR_B_PORT PORTD
#define RAM_CTR_B_PIN 7
#define RAM_CTR_B_DDR DDRD

//Bluetooth
#define BT_RADIO_TYPE ERICSSON
#define BT_RADIO_ON_PORT PORTF
#define BT_RADIO_ON_PIN 0
#define BT_RADIO_ON_DDR DDRF
#define BT_POWER_ON_PORT PORTF
#define BT_POWER_ON_PIN 2
#define BT_POWER_ON_DDR DDRF
#define BT_RESET_PORT PORTF
#define BT_RESET_PIN 1
#define BT_RESET_DDR DDRF

// Other ADCs
#define BAT_SENSE_ADC           3
#define BAT_DIVIDER             2

/* Initial speed for the uart */
#define BT_UART_INITIAL_SPEED 57600
// Parity for the uart (0-None, 1-Odd, 2-Even)
#define BT_UART_INITIAL_PARITY 0
#define BT_UART_INITIAL_DATABITS 8
#define BT_UART_INITIAL_STOPBITS 1
// Flowcontrol to use (UART_HS_RTSCTS or 0 for none)
#define BT_UART_INITIAL_FLOWCONTROL UART_HS_RTSCTS


//I2C
// master trigger
#define I2C_MASTER_TRIGGER_PORT PORTD
#define I2C_MASTER_TRIGGER_PIN 6
#define I2C_MASTER_TRIGGER_DDR DDRD

//LED's
// we have LEDs on PORT B
#define LED_PORT PORTB
#define LED_PORT_DDR DDRB
#define LED0    4
#define LED1    5
#define LED2    6
#define LED3    7

//define bt uart and app uart
#define APP_UART devUsartAvr1
#define APP_UART_NR 1
#define BT_UART devUsartAvr0
#define BT_UART_NR 0

//Uart0
// RxD: PE0
// TxD: PE1
// RTS: PE2
// CTS: PE4 (external interrupt 4)
#define UART0_CTS_PIN PINE      // PIN where CTS is connected
#define UART0_CTS_PORT PORTE    // PORT where CTS is connected
#define UART0_CTS_DDR DDRE      // DataDirectionRegister of CTS PORT
#define UART0_CTS_BIT 4         // Pin on which CTS is connected
#define UART0_CTS_SIGNAL sig_INTERRUPT4   // Interrupt triggered by CTS
#define UART0_RTS_PORT PORTE
#define UART0_RTS_DDR DDRE
#define UART0_RTS_BIT 2

// BT-UART latency optimization:
// - hardcode uart irq handler to usartavr
#define USE_USART0
// - read all bytes of uart rx buffer
#define UART0_READMULTIBYTE
// - strip code to handle software flowcontrol
#define UART0_NO_SW_FLOWCONTROL


//Uart1
// RxD: PD2
// TxD: PD3
// RTS: PD4
// CTS: PD5
#define UART1_CTS_PIN PIND      // PIN where CTS is connected
#define UART1_CTS_PORT PORTD    // PORT where CTS is connected
#define UART1_CTS_DDR DDRD      // DataDirectionRegister of CTS PORT
/* 
 * @note: Commented the following line, because Nut/OS otherwise would try to regiser a
 * CTS interrupt (which is not possible on btnode2). 
 * In the BTnode System drivers CTS handshake was also not implemented
 */
// #define UART1_CTS_BIT 5         // Pin on which CTS is connected
// #define UART1_CTS_SIGNAL        // Interrupt triggered by CTS (none)
#define UART1_RTS_PORT PORTD
#define UART1_RTS_DDR DDRD
#define UART1_RTS_BIT 4

//software UART

//new defines and variables for software uart
#define sig_SUART_RECV     sig_INTERRUPT6 //PE6

#define SUART_RXD_PORT PORTE    // PORT where RXD is connected
#define SUART_RXD_PIN PINE
#define SUART_RXD 6             // Pin on which RXD is connected
#define SUART_RXD_INTERRUPT INT6
#define SUART_TXD_PORT PORTE
#define SUART_TXD_PORT_DDR DDRE
#define SUART_TXD 5





