/*
 * Copyright (C) 2000-2005 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch/
 */

/**
 *
 * \file cm_chain.h
 *
 * \author Matthias Ringwald
 *
 * \brief Simple connection manager to establish pre-defined chain topology
 *
 * LED CODE:
 * LED 0 (blue): Previous node has connected to me
 * LED 1 (red):  Connected to next node
 */

#include <sys/types.h>
#include <bt/bt_hci_defs.h>
#include <bt/bt_hci_cmds.h>

/**
 * get position of node with addr
 *
 * \param addr of node
 * \returns position in chain, indexed by 0
 */
u_char connect_get_position_of(bt_addr_t addr);

/**
 * get position of own node
 *
 * \param addr of node
 * \returns position in chain, indexed by 0
 */
u_short connect_get_position(void);

/**
 * Init Chain Connection Manager
 *
 * \param struct btstack
 * \param topo array of node addresses
 * \param number of nodes in chain
 *
 */

void connect_init( struct btstack *stack, bt_addr_t* topo, u_short length);
